function [sys,x0,str,ts] = ballbeamdyn(t,x,u,flag)

% BALL on BEAM DYNAMICS S-function
%
% theta    -------   position
%     --> |  BnB  |-->
%          -------
%
% 2007 Joo Paulo Coelho
%
%   www.ipb.pt/~jpcoelho

%----------------------------------------------------------------------------
% A cada chamada o SIMULINK utiliza flag para diferenciar o modo de execuo
%----------------------------------------------------------------------------
switch flag,
	case 0,                          % --> Inicializao: Definio das condies iniciais
      sizes = simsizes;                % Constroi, automaticamente, uma estrutura a preencher pelo programador
   		sizes.NumContStates  = 2;     % Nmero de estados na representao em espao de estados..
   		sizes.NumDiscStates  = 0;     % The number of discrete states
   		sizes.NumOutputs     = 1;     % The number of outputs
   		sizes.NumInputs      = 1;     % The number of inputs
   		sizes.DirFeedthrough = 0;     % The direct feedthrough flag.
      	sizes.NumSampleTimes = 1;     % Nmero de amotras num determinado instante de tempo (1 significa que a cada iterao apenas ser apresentada a sada referente a esse instante)
   	sys = simsizes(sizes);           % Converte estrutura em um vector linha...
      % Inicializa condies iniciais (posio inicial da bola e sua primeira derivada)
      nome_pai = get_param(gcs,'Parent');
      x_ball=str2num(get_param([nome_pai '/Ball on Beam'],'x_Ball'));
      v_ball=str2num(get_param([nome_pai '/Ball on Beam'],'v_Ball'));
      if isempty(x_ball)|length(x_ball)>1|ischar(x_ball),x_ball=0;end
      if isempty(v_ball)|length(v_ball)>1|ischar(v_ball),v_ball=0;end
		x0  = [x_ball v_ball ];
		% str  sempre uma matriz vazia (reservada para futuras alteraes...)
		str = [];
   	% Define perodo de amostragem e offset (modelo analgico => Ts=0)
   	ts  = [0 0];  
      % Opes de simulao .... UFA!
      OpSim=get_param(gcs,'Parent');
      set_param(OpSim,'Solver','ode4','FixedStep','.0242','StopTime','60');
      % Abre a Janela (se ainda no estiver aberta) e inicializa GUI sistema...
      InicializaSistema(x_ball,v_ball);
   case 1, 
      % Clculos envolvendo os vectores de entrada e clculo das derivadas
      Fig = get_param(gcs,'UserData');
      HndStruc=get(Fig,'UserData');
      csetup=HndStruc.csetup;
      m=csetup.m;
      r=csetup.r;
      g=csetup.g;
      l=csetup.l;
      lb=csetup.lb;
      cf=csetup.cf;
      cdist=csetup.cdist;

      sys = ballbeamode(t,x,[],u,r,m,g,l,lb,cf); % Actualizao da posio da bola e calha
      sys=sys+cdist;
      ActualizaSistema(t,x,u);
      
      if cdist(1)~=0,
         csetup.cdist=[0;0];
         HndStruc.csetup=csetup;
         set(Fig,'UserData',HndStruc);
      end
      
	case 2    
      sys=[];
   case 3,
      sys=x(1);  %retorna posio da bola...
      %ActualizaSistema(t,x,u);
   case {4,9},
      Fig = get_param(gcs,'UserData');
      set(Fig,'CloseRequestFcn','closereq');
      sys=[];
   case 'LeftDisturb'
      Fig = get_param(gcs,'UserData');
      HndStruc=get(Fig,'UserData');
      csetup=HndStruc.csetup;
      Disturb=HndStruc.Disturb;
      csetup.cdist=[-rand*str2num(get(Disturb,'String'))*2*csetup.l;0];
      HndStruc.csetup=csetup;
      set(Fig,'UserData',HndStruc);
      set_param(gcs,'UserData',Fig);
   case 'RightDisturb'
      Fig = get_param(gcs,'UserData');
      HndStruc=get(Fig,'UserData');
      csetup=HndStruc.csetup;
      Disturb=HndStruc.Disturb;
      csetup.cdist=[rand*str2num(get(Disturb,'String'))*2*csetup.l;0];
      HndStruc.csetup=csetup;
      set(Fig,'UserData',HndStruc);
      set_param(gcs,'UserData',Fig);
	otherwise
		error(['Unhandled flag = ',num2str(flag)]);
end

%----------------------------------------------------------------------------------------------------------------------------------
function InicializaSistema(x_ball,v_ball)

Fig = get_param(gcs,'UserData');
if ishandle(Fig),
   HndStruc=get(Fig,'UserData');
   Eixo=HndStruc.Eixo;
   Bola=HndStruc.Bola;
   Balance=HndStruc.Balace;
   Angulo=HndStruc.Angulo;
   Posicao=HndStruc.Posicao;
   Velocidade=HndStruc.Velocidade;
   csetup=HndStruc.csetup;
   %-----------------------------------------------------
   %       ACTUALIZA BALANC
   %-----------------------------------------------------
   L=csetup.l*csetup.fescala;       % L - Comprimento do Brao
   A=csetup.a*csetup.fescala;       % A - Largura do Brao
   theta=csetup.theta;
   pos=csetup.x;
   xo=L*cos(theta);yo=L*sin(theta); % xo e yo - Posio do segmento director com base no angulo
   dx=A*sin(theta);dy=A*cos(theta); % dx e dy - Diferenciais requeridos para modelar a largura do brao
   set(Balance,'XData',[-(xo+dx/2) -(xo-dx/2);xo-dx/2 xo+dx/2],'YData',[yo-dy/2 yo+dy/2;-(yo+dy/2) -(yo-dy/2)]);
   %-----------------------------------------------------
   %       ACTUALIZA BOLA
   %-----------------------------------------------------
   set(Bola,...
      'XData',csetup.fescala*(csetup.r*cos(0:pi/100:2*pi)+pos),...
      'YData',csetup.fescala*(csetup.r*sin(0:pi/100:2*pi)+((csetup.r+csetup.a/2)/cos(theta)-tan(theta)*pos)));
   drawnow;
   figure(Fig);
else
   % CARACTERSTICAS DO SISTEMA...
	% m  - massa da bola
	% g  - acelerao da gravidade
	% r  - raio da bola
	% l  - comprimento da calha
   % Ib - momento de inrcia da bola
   csetup.m       = .22;                        % massa da bola (Kg)
   csetup.r       = .04;                        % raio da bola (m)
   csetup.g       = 9.8;                        % acelerao gravidade (m.s^-2)
   csetup.l       = .6;                         % 1/2 comprimento da calha (m)
   csetup.a       = .02;                        % largura da calha (m)     
   csetup.lb      = (2/5)*csetup.m*csetup.r^2;  % momento de inecia da bola (Kg.m^2)
   csetup.cf      = 1.81e-5;                    % coeficiente de frico
   csetup.fescala = 33;                         % factor de escala
   csetup.theta   = 0;                          % angulo Inicial (tomado em referncia ao semi-eixo negativo das ordenadas) (rad)
   csetup.offset  = .003;                       % distncia de contacto da bola com a calha (m)
   csetup.veloc   = v_ball;
   csetup.x       = x_ball;                     % posio inicial (m)
   csetup.y       = (csetup.r+csetup.a/2)/...
      				   cos(csetup.theta)-...
      					tan(csetup.theta)*csetup.x;
   csetup.time    = 0;                          % tempo inicial
   csetup.f       = .25;                        % altura do fulcro (m)
   csetup.p       = .2;                         % largura da base  (m) 
   csetup.cdist   = [0;0];					  		   % perturbao presente...
   
   Fig=ballbeamgui(csetup);
	set_param(gcs,'UserData',Fig);               % Guarda o handle da figura
end
%----------------------------------------------------------------------------------------------------------------------------------
function ActualizaSistema(t,x,u)

Fig = get_param(gcs,'UserData');

HndStruc=get(Fig,'UserData');
Eixo=HndStruc.Eixo;
Bola=HndStruc.Bola;
Balance=HndStruc.Balace;
Angulo=HndStruc.Angulo;
Posicao=HndStruc.Posicao;
Velocidade=HndStruc.Velocidade;

theta=u;
pos=x(1);
vel=x(2);

csetup=HndStruc.csetup;
%-----------------------------------------------------
%       ACTUALIZA BALANC
%-----------------------------------------------------
L=csetup.l*csetup.fescala;       % L - Comprimento do Brao
A=csetup.a*csetup.fescala;       % A - Largura do Brao
xo=L*cos(theta);yo=L*sin(theta); % xo e yo - Posio do segmento director com base no angulo
dx=A*sin(theta);dy=A*cos(theta); % dx e dy - Diferenciais requeridos para modelar a largura do brao
set(Balance,'XData',[-(xo+dx/2) -(xo-dx/2);xo-dx/2 xo+dx/2],'YData',[yo-dy/2 yo+dy/2;-(yo+dy/2) -(yo-dy/2)]);
%-----------------------------------------------------
%       ACTUALIZA BOLA
%-----------------------------------------------------
set(Bola,...
   'XData',csetup.fescala*(csetup.r*cos(0:pi/100:2*pi)+pos),...
   'YData',csetup.fescala*(csetup.offset+csetup.r*sin(0:pi/100:2*pi)+((csetup.r+csetup.a/2)/cos(theta)-tan(theta)*pos)));
%-----------------------------------------------------
%       ACTUALIZA INFO
%-----------------------------------------------------
set(Posicao,'String',num2str(100*pos));
set(Angulo,'String',num2str(theta*180/pi));
set(Velocidade,'String',num2str(vel));
drawnow; 